# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'Configuration.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_ConfDialog(object):
    def setupUi(self, ConfDialog):
        ConfDialog.setObjectName("ConfDialog")
        ConfDialog.resize(695, 449)
        self.horizontalLayout = QtWidgets.QHBoxLayout(ConfDialog)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.confPrefLayout = QtWidgets.QVBoxLayout()
        self.confPrefLayout.setSpacing(1)
        self.confPrefLayout.setObjectName("confPrefLayout")
        self.confPrefLabel = QtWidgets.QLabel(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confPrefLabel.sizePolicy().hasHeightForWidth())
        self.confPrefLabel.setSizePolicy(sizePolicy)
        self.confPrefLabel.setMinimumSize(QtCore.QSize(0, 25))
        self.confPrefLabel.setMaximumSize(QtCore.QSize(16777215, 25))
        self.confPrefLabel.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.confPrefLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.confPrefLabel.setObjectName("confPrefLabel")
        self.confPrefLayout.addWidget(self.confPrefLabel)
        self.confPrefList = MixListWidget(ConfDialog)
        self.confPrefList.setDragDropMode(QtWidgets.QAbstractItemView.NoDragDrop)
        self.confPrefList.setObjectName("confPrefList")
        self.confPrefLayout.addWidget(self.confPrefList)
        self.horizontalLayout.addLayout(self.confPrefLayout)
        self.confPassiveLayout = QtWidgets.QVBoxLayout()
        self.confPassiveLayout.setSpacing(1)
        self.confPassiveLayout.setObjectName("confPassiveLayout")
        self.confPassiveTopLayout = QtWidgets.QHBoxLayout()
        self.confPassiveTopLayout.setSpacing(0)
        self.confPassiveTopLayout.setObjectName("confPassiveTopLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.confPassiveTopLayout.addItem(spacerItem)
        self.confPassiveLabel = QtWidgets.QLabel(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confPassiveLabel.sizePolicy().hasHeightForWidth())
        self.confPassiveLabel.setSizePolicy(sizePolicy)
        self.confPassiveLabel.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.confPassiveLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.confPassiveLabel.setObjectName("confPassiveLabel")
        self.confPassiveTopLayout.addWidget(self.confPassiveLabel)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.confPassiveTopLayout.addItem(spacerItem1)
        self.confPassiveAddButton = QtWidgets.QPushButton(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confPassiveAddButton.sizePolicy().hasHeightForWidth())
        self.confPassiveAddButton.setSizePolicy(sizePolicy)
        self.confPassiveAddButton.setMinimumSize(QtCore.QSize(25, 25))
        self.confPassiveAddButton.setMaximumSize(QtCore.QSize(25, 25))
        self.confPassiveAddButton.setAutoDefault(False)
        self.confPassiveAddButton.setDefault(False)
        self.confPassiveAddButton.setFlat(False)
        self.confPassiveAddButton.setObjectName("confPassiveAddButton")
        self.confPassiveTopLayout.addWidget(self.confPassiveAddButton)
        self.confPassiveDelButton = QtWidgets.QPushButton(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confPassiveDelButton.sizePolicy().hasHeightForWidth())
        self.confPassiveDelButton.setSizePolicy(sizePolicy)
        self.confPassiveDelButton.setMinimumSize(QtCore.QSize(25, 25))
        self.confPassiveDelButton.setMaximumSize(QtCore.QSize(25, 25))
        self.confPassiveDelButton.setAutoDefault(False)
        self.confPassiveDelButton.setObjectName("confPassiveDelButton")
        self.confPassiveTopLayout.addWidget(self.confPassiveDelButton)
        self.confPassiveLayout.addLayout(self.confPassiveTopLayout)
        self.confPassiveList = MixListWidget(ConfDialog)
        self.confPassiveList.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.confPassiveList.setObjectName("confPassiveList")
        self.confPassiveLayout.addWidget(self.confPassiveList)
        self.horizontalLayout.addLayout(self.confPassiveLayout)
        self.confActiveLayout = QtWidgets.QVBoxLayout()
        self.confActiveLayout.setSpacing(1)
        self.confActiveLayout.setObjectName("confActiveLayout")
        self.confActiveTopLayout = QtWidgets.QHBoxLayout()
        self.confActiveTopLayout.setSpacing(0)
        self.confActiveTopLayout.setObjectName("confActiveTopLayout")
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.confActiveTopLayout.addItem(spacerItem2)
        self.confActiveLabel = QtWidgets.QLabel(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confActiveLabel.sizePolicy().hasHeightForWidth())
        self.confActiveLabel.setSizePolicy(sizePolicy)
        self.confActiveLabel.setLayoutDirection(QtCore.Qt.LeftToRight)
        self.confActiveLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.confActiveLabel.setObjectName("confActiveLabel")
        self.confActiveTopLayout.addWidget(self.confActiveLabel)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.confActiveTopLayout.addItem(spacerItem3)
        self.confActiveAddButton = QtWidgets.QPushButton(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confActiveAddButton.sizePolicy().hasHeightForWidth())
        self.confActiveAddButton.setSizePolicy(sizePolicy)
        self.confActiveAddButton.setMinimumSize(QtCore.QSize(25, 25))
        self.confActiveAddButton.setMaximumSize(QtCore.QSize(25, 25))
        self.confActiveAddButton.setAutoDefault(False)
        self.confActiveAddButton.setObjectName("confActiveAddButton")
        self.confActiveTopLayout.addWidget(self.confActiveAddButton)
        self.confActiveDelButton = QtWidgets.QPushButton(ConfDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.confActiveDelButton.sizePolicy().hasHeightForWidth())
        self.confActiveDelButton.setSizePolicy(sizePolicy)
        self.confActiveDelButton.setMinimumSize(QtCore.QSize(25, 25))
        self.confActiveDelButton.setMaximumSize(QtCore.QSize(25, 25))
        self.confActiveDelButton.setAutoDefault(False)
        self.confActiveDelButton.setObjectName("confActiveDelButton")
        self.confActiveTopLayout.addWidget(self.confActiveDelButton)
        self.confActiveLayout.addLayout(self.confActiveTopLayout)
        self.confActiveList = MixListWidget(ConfDialog)
        self.confActiveList.setObjectName("confActiveList")
        self.confActiveLayout.addWidget(self.confActiveList)
        self.horizontalLayout.addLayout(self.confActiveLayout)

        self.retranslateUi(ConfDialog)
        QtCore.QMetaObject.connectSlotsByName(ConfDialog)

    def retranslateUi(self, ConfDialog):
        _translate = QtCore.QCoreApplication.translate
        ConfDialog.setWindowTitle(_translate("ConfDialog", "Configuration"))
        self.confPrefLabel.setText(_translate("ConfDialog", "Preferences"))
        self.confPrefList.setSortingEnabled(True)
        self.confPassiveLabel.setText(_translate("ConfDialog", "Passive Actions Ordering"))
        self.confPassiveAddButton.setText(_translate("ConfDialog", "+"))
        self.confPassiveDelButton.setText(_translate("ConfDialog", "-"))
        self.confActiveLabel.setText(_translate("ConfDialog", "Active Actions"))
        self.confActiveAddButton.setText(_translate("ConfDialog", "+"))
        self.confActiveDelButton.setText(_translate("ConfDialog", "-"))
        self.confActiveList.setSortingEnabled(True)

from CustomWidgets import MixListWidget
